package c64.tape;

import java.io.FileInputStream;
import java.io.IOException;
import c64.tape.ex.EndOfTapFileException;

public class C64TapFile{
	private C64TapImage image;
	private String tapname;
	private static byte signature[] = new byte[ 20 ];
	private static byte version;
	private static final String MAGIC_LABEL = "C64-TAPE-RAW";
	private static final byte CURRENT_MAX_TAP_VERSION = 1;
	private static final byte[] FUTURE_TAP_EXPANSION = { 0x00, 0x00 };

	public C64TapFile( String path )
		throws IOException{

			tapname = path;
			image = new C64TapImage( path, signature );
	}

	public static boolean seekForValidTapFile( String name )
		throws IOException{
			FileInputStream inputfile = new FileInputStream( name );
			int code = inputfile.read( signature );
			inputfile.close( );

			return
				( code != -1 ) 											 &
				new String( signature, 0, 12 ).equals( MAGIC_LABEL )     &
				( version = signature[ 12 ] ) <= CURRENT_MAX_TAP_VERSION &
				signature[ 14 ] == FUTURE_TAP_EXPANSION[ 0 ]             &
				signature[ 15 ] == FUTURE_TAP_EXPANSION[ 1 ];
	}

	public C64TapImage getTapImage( ){
		return image;
	}

	public byte readFromTapFile( )
		throws EndOfTapFileException{
			int b = image.read( );
			if( b < 0 )
				throw new EndOfTapFileException( );
			return (byte)b;
	}

	public byte[] readBytesFromTapFile( int n )
		throws EndOfTapFileException{
			byte[] bundle = new byte[ n ];
			if( image.read( bundle, n ) < 0 )
				throw new EndOfTapFileException( );
			return bundle;
	}

	public int getTapFileOffset( )
	{
		return image.getCrsr( );
	}

	public void saveData( )
		throws IOException
	{
		image.saveData( );
	}

	public void close( )
	{
		image.close( );
	}
}
