package c64.tape;

import gui.Messages;
import conf.Configuration;

import java.util.Vector;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

public class C64TapImage{
	private String filePath;
	private byte[] tapimage;
	private byte[] signature;
	private int tapimagesize;
	private Vector C64TapPrgs;
	private int cursor;

	public C64TapImage( String file, byte[] sig )
		throws IOException{

			filePath = file;
			C64TapPrgs = new Vector( );
			setTapImage( file, sig );
	}

	void addTapImagePrg( C64TapPrg prg ){
		C64TapPrgs.add( prg );
	}

	void addTapImagePrg( int i, C64TapPrg prg ){
		C64TapPrgs.add( i, prg );
	}

	void removeTapImagePrg( int i ){
		C64TapPrgs.remove( i );
	}

	public C64TapPrg getTapImagePrg( int i ){
		return ( C64TapPrg )C64TapPrgs.get( i );
	}

	public int getTapImagePrgsCount( ){
		return C64TapPrgs.size( );
	}

	void changeTapImageSize( long sz ){
		tapimagesize += sz;

		if( tapimagesize > tapimage.length )
		{
			Object temp = tapimage;
			tapimage = new byte[ tapimagesize ];
			System.arraycopy( temp, 0, (Object)tapimage, 0, (int)(tapimagesize - sz) );
		}
	}

	long getTapImageSize( ){
		return tapimagesize;
	}

	public void addPrgToTapImage( C64TapPrg prg, int mode ){
		switch( mode & 0x80 ){
			//aggiunge nel mezzo
			case C64TapImageMutator.INTRA:
				C64TapImageMutator.addIntra( this, prg, mode & 0x7F );
				break;
			//aggiunge in coda
			case C64TapImageMutator.TRAIL:
				C64TapImageMutator.addToTrail( this, prg );
				break;
		}
	}

	public void deletePrgFromTapImage( int mode ){
		switch( mode & 0x80 ){
			//cancella nel mezzo
			case C64TapImageMutator.INTRA:
				C64TapImageMutator.deleteIntra( this, mode & 0x7F );
				break;
			//cancella la coda
			case C64TapImageMutator.TRAIL:
				C64TapImageMutator.deleteTrail( this );
				break;
		}
	}

	public byte[] getTapImageData( ){
		return tapimage;
	}

	public int read( )
	{
		if( cursor == tapimagesize )
			return -1;
		return tapimage[ cursor++ ] & 0x000000FF;
	}

	public int read( byte[] data, int n )
	{
		if( n > tapimagesize - cursor )
			return -1;
		System.arraycopy( (Object)tapimage, cursor, (Object)data, 0, n );
		 cursor += n;
			return  0;
	}

	public int getCrsr()
	{
		return cursor;
	}

	public void saveData( )
		throws IOException{

			String newFilePath;
			File file;

			scan:
			while( ( file = new java.io.File( filePath ) ).exists( ) && !file.canWrite( ) )
			{
				switch( Messages.showOverwriteConfirm( filePath ) )
				{
					//non fare nulla
					case javax.swing.JOptionPane.CANCEL_OPTION :
						return;
					//non fare nulla
					case javax.swing.JOptionPane.CLOSED_OPTION :
						return;
					//sproteggilo e prosegui
					case javax.swing.JOptionPane.YES_OPTION :
					{
						file.delete( );
						break scan;
					}
					case javax.swing.JOptionPane.NO_OPTION :
						filePath = Messages.getFilePath( );
					break;
				}
			}

			Configuration.tapfileName = file.getName( );

			FileOutputStream fos = new FileOutputStream( filePath );
			int datasize   = tapimagesize - signature.length;
			tapimage[ 16 ] = (byte) datasize;
			tapimage[ 17 ] = (byte)(datasize >>>  8);
			tapimage[ 18 ] = (byte)(datasize >>> 16);
			tapimage[ 19 ] = (byte)(datasize >>> 24);
			fos.write( tapimage, 0, tapimagesize );
			fos.close( );
	}

	public void close( )
	{
		cursor = 0;
	}

	private void setTapImage( String file, byte[] sig )
		throws IOException{

			FileInputStream fis = new FileInputStream( file );
			tapimagesize        = fis.available( );
			tapimagesize        = tapimagesize + (( sig[ 13 ] == 0 ) ? C64TapImageMutator.PADDINGSIZE : 0 );

			tapimage = new byte[ tapimagesize ];
			fis.read( tapimage );
			fis.close( );

			if( sig[ 13 ] == 0 )
				tapimage[ 13 ] = 1;

			signature           = sig;
	}
}
