package c64.tape;

import java.io.IOException;
import c64.tape.ex.EndOfTapFileException;

public class KernelLoader implements Loader{
	private C64TapFile tapfile;
	private byte FIRST_THRESHOLD  = (byte)0x38;
	private byte SECOND_THRESHOLD = (byte)0x4D;
	static int BACKWARD=182,TOREADAFTERSINCRO=420;
	private byte[] sincro=
	{ (byte)0x89, (byte)0x88, (byte)0x87,
	  (byte)0x86, (byte)0x85, (byte)0x84,
	  (byte)0x83, (byte)0x82, (byte)0x81 };
	private byte[] prgname=new byte[16];
	private	byte[] array=new byte[TOREADAFTERSINCRO];
	private int prgtype;

	public KernelLoader(C64TapFile file){
		tapfile=file;
	}

	public long getHeader()throws IOException,EndOfTapFileException{
		int read;

		if(tapfile==null)
			return -1;

		do{
			do{
				while(getPulseType(tapfile.readFromTapFile())!=Pulse.LONG);
				while((read=getPulseType(tapfile.readFromTapFile()))==Pulse.LONG);
			}while(read==Pulse.SHORT);

		}while(searchForHeader()==-1);

		return tapfile.getTapFileOffset()-BACKWARD;
	}

	public long getData()throws IOException,EndOfTapFileException{
		return (long)prgtype;
	}

	public String getName()throws IOException,EndOfTapFileException{

		array=tapfile.readBytesFromTapFile(TOREADAFTERSINCRO);

		prgtype=	 getByte(array,  0);
		prgname[0]=  getByte(array,100);
		prgname[1]=  getByte(array,120);
		prgname[2]=  getByte(array,140);
		prgname[3]=  getByte(array,160);
		prgname[4]=  getByte(array,180);
		prgname[5]=  getByte(array,200);
		prgname[6]=  getByte(array,220);
		prgname[7]=  getByte(array,240);
		prgname[8]=  getByte(array,260);
		prgname[9]=  getByte(array,280);
		prgname[10]= getByte(array,300);
		prgname[11]= getByte(array,320);
		prgname[12]= getByte(array,340);
		prgname[13]= getByte(array,360);
		prgname[14]= getByte(array,380);
		prgname[15]= getByte(array,400);

		return new String(prgname);
	}

	private int searchForHeader()throws IOException,EndOfTapFileException{
		if(!checkForHeader(tapfile.readBytesFromTapFile(180)))
			return -1;
		return 0;
	}

	private boolean checkForHeader(byte[] b){
		return
			(getByte(b,0)   == sincro[0]) &&
			(getByte(b,20)  == sincro[1]) &&
			(getByte(b,40)  == sincro[2]) &&
			(getByte(b,60)  == sincro[3]) &&
			(getByte(b,80)  == sincro[4]) &&
			(getByte(b,100) == sincro[5]) &&
			(getByte(b,120) == sincro[6]) &&
			(getByte(b,140) == sincro[7]) &&
			(getByte(b,160) == sincro[8]);
	}

	private byte getByte(byte[] b, int start){
		int data=0;

		for(int i=0;i<8;i++){
			data>>>=1;
		    if((getPulseType(b[start++])==Pulse.MEDIUM) & (getPulseType(b[start++])==Pulse.SHORT))
		    	data|=0x80;
		}
		return (byte)data;
	}

	private int getPulseType(byte b){
		if (b < FIRST_THRESHOLD)
			return Pulse.SHORT;
		if ((FIRST_THRESHOLD <= b) && (b < SECOND_THRESHOLD))
			return Pulse.MEDIUM;
		return Pulse.LONG;
	}

	private class Pulse{
		final static int SHORT=1;
		final static int MEDIUM=2;
		final static int LONG=3;
	}
}
