package gui;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.util.*;

import c64.tape.*;
import conf.Configuration;

class CustomPanel extends JPanel{
	private JFrame parent;
	private JTextField textfield;
	private JList textarea;
	private JButton button;
	private C64TapFile tapfile;
	private Vector prgNameVector;
	private int srcInd = -1;
	//private int id;
	static boolean firstTime = true;

	CustomPanel( JFrame parent ){
		this.parent = parent;
		setLayout( new BorderLayout( ) );
		textfield = new JTextField( );
		textfield.setEditable( false );
		textarea = new JList( );
		JScrollPane scrollPane = new JScrollPane( textarea );
		JPanel panel = new JPanel( );
		panel.setLayout( new BorderLayout( ) );
		panel.add( new JLabel( "Programmi nel nastro" ), BorderLayout.NORTH );
		panel.add( scrollPane , BorderLayout.CENTER );
		button = new JButton( "Sfoglia" );
		add( textfield, BorderLayout.NORTH );
		add( panel, BorderLayout.CENTER );
		add( button, BorderLayout.SOUTH );
		addListeners( );
	}

	private void addListeners( ){

		DragSource ds = DragSource.getDefaultDragSource( );
		ds.createDefaultDragGestureRecognizer( textarea, DnDConstants.ACTION_COPY_OR_MOVE,
			new CustomDragGestureListener( ) );
		DropTarget dt = new DropTarget( textarea, new CustomDropTargetListener( ) );
		dt.setDefaultActions( DnDConstants.ACTION_COPY_OR_MOVE );

		button.addActionListener(
			new ActionListener( ){
				public void actionPerformed(ActionEvent e){
					String path;

					try{
						if( firstTime )
						{
							if( new java.io.File( Configuration.configFileName ).exists( ) )
								path = Configuration.readCurrentDirFromCfgFile( );
							else
								path = Configuration.currentDir;

								firstTime = false;
						}
						else
							path = Configuration.currentDir;

						JFileChooser chooser = new JFileChooser( path );

						if( chooser.showOpenDialog( parent ) == JFileChooser.APPROVE_OPTION )
						{
							String selected = chooser.getSelectedFile( ).getPath( );
							if( C64TapFile.seekForValidTapFile( selected ) == true ){

								if( ((MainFrame)parent).isTheSameFile( CustomPanel.this, selected ) )
									return;

								textfield.setText( chooser.getSelectedFile( ).getName( ) );
								textarea.removeAll( );

								tapfile = new C64TapFile( selected );
								C64TapProcessor proc = new C64TapProcessor( tapfile );
								proc.searchForPrgs( );
								C64TapImage image = tapfile.getTapImage( );

								prgNameVector = new Vector( );

								for( int j=0; j<image.getTapImagePrgsCount( ); j++ )
									prgNameVector.add( image.getTapImagePrg( j ).getName( ) );

								textarea.setListData( prgNameVector );
							}
							Configuration.currentDir = selected;
						}
					}

					catch( Exception ex ){
						ex.printStackTrace( );
					}
				}
			}
		);

		textarea.addMouseListener( new CustomMouseAdapter( ) );
	}

	private class CustomMouseAdapter extends MouseAdapter{

		public void mouseClicked( MouseEvent e )
		{
			if( tapfile != null && prgNameVector.size() > 0 ){
				if ( e.getClickCount( ) == 2 )
				{
					int indexToDelete = textarea.locationToIndex( e.getPoint( ) );
					tapfile.getTapImage( ).deletePrgFromTapImage( C64TapImageMutator.INTRA | indexToDelete );
					prgNameVector.remove( indexToDelete );
					textarea.setListData( prgNameVector );
				}
			}
		}
	}

	private class CustomDragGestureListener implements DragGestureListener{

		public void dragGestureRecognized( DragGestureEvent event ){

			if( tapfile == null )
				return;

			if( prgNameVector.size() == 0 )
				return;

			C64TapImageMutator.setSourceData( tapfile.getTapImage( ).getTapImageData( ) );

			textarea.setSelectedIndex( srcInd = textarea.locationToIndex(event.getDragOrigin( ) ) );
			event.startDrag( null, new Dummy( textarea.getSelectedIndex( ) ),
				new CustomDragSourceListener( ) );

		}
	}
	//oggetto che ingloba i dati da trasmettere all'oggetto DropTarget
	private class Dummy implements Transferable{

		private C64TapPrg prgObj;

		private final DataFlavor[] flavors={
			DataFlavor.stringFlavor,
		};

		Dummy( int index ){
			prgObj = tapfile.getTapImage( ).getTapImagePrg( index );
		}

		public boolean isDataFlavorSupported( DataFlavor flavor ){
			return false;
		}

		public DataFlavor[] getTransferDataFlavors( ){
			return flavors;
		}

		public Object getTransferData( DataFlavor flavor ){
			return prgObj;
		}
	}

	private class CustomDragSourceListener implements DragSourceListener{

		public void dragEnter( DragSourceDragEvent dsde ){
			//System.out.println( "Uscito da source ed entrato in target (notificato da "+ id +")");
		}

		public void dragExit( DragSourceEvent dse ){
			//System.out.println( "Uscito da target (notificato da "+ id +")" );
		}

		public void dragOver( DragSourceDragEvent dsde ){
			//System.out.println( "In movimento su target (notificato da "+ id +")" );
		}

		public void dragDropEnd( DragSourceDropEvent dsde ){
			//System.out.println( "Operazione completata (notificato da "+ id +")" );
			if( ((MainFrame)parent).isMoveActive() && dsde.getDropSuccess() )
			{
				tapfile.getTapImage( ).deletePrgFromTapImage( C64TapImageMutator.INTRA | srcInd );

				prgNameVector.remove( srcInd );
				textarea.setListData( prgNameVector );
			}
			srcInd = -1;
		}

		public void dropActionChanged( DragSourceDragEvent dsde ){ }
	}

	private class CustomDropTargetListener implements DropTargetListener{

		public void dragEnter( DropTargetDragEvent dtde ){
			//System.out.println( "Entrato in questo target (notificato da "+ id +")" );
		}

		public void dragExit( DropTargetEvent dte ){
			//System.out.println( "Uscito da questo target (notificato da "+ id +")" );
		}

		public void dragOver( DropTargetDragEvent dtde ){
			//System.out.println( "In movimento su questo target (notificato da "+ id +")" );
		}

		public void drop( DropTargetDropEvent dtde ){
			//System.out.println( "Operazione completata su questo target (notificato da "+ id +")" );

			if( tapfile == null )
				dtde.rejectDrop( );

			else
			{
				Point pt = dtde.getLocation( );
				Rectangle rect = (prgNameVector.size() == 0) ? new Rectangle( ) : textarea
					.getCellBounds( 0, prgNameVector.size() - 1 );

				int	index = pt.getY( ) > rect.getY( ) + rect.getHeight( ) ?
					prgNameVector.size( ) : textarea.locationToIndex( pt );

				Transferable t = dtde.getTransferable( );
				DataFlavor[] flavors = t.getTransferDataFlavors( );

				try
				{
					C64TapPrg tapPrg  = (C64TapPrg)t.getTransferData( flavors[ 0 ] );
					String tapPrgName = tapPrg.getName( );

					prgNameVector.add( index, tapPrgName );
					textarea.setListData( prgNameVector );

					tapfile.getTapImage( ).addPrgToTapImage( tapPrg, index == prgNameVector.size( )
						- 1 ? C64TapImageMutator.TRAIL : C64TapImageMutator.INTRA | index );

					if(((MainFrame)parent).isMoveActive() && srcInd != -1 && index < srcInd)
						srcInd++;

					dtde.dropComplete( true );
				}

				catch( Exception ex ){
					ex.printStackTrace( );
				}
			}
		}

		public void dropActionChanged( DropTargetDragEvent dtde ){ }
	}

	void commitChanges( )
	{
		try
		{
			if(tapfile != null)
			{
				tapfile.saveData( );
				textfield.setText( Configuration.tapfileName );
			}
		}
		catch(java.io.IOException ex){ ex.printStackTrace(); }
	}
}
